@@ -5,16 +5,16 @@
 */
 
 #include "kurlnavigatortest.h"
-#include <QtTestWidgets>
+#include <KFilePlacesModel>
+#include <KUser>
 #include <QDir>
 #include <QPushButton>
 #include <QStandardPaths>
-#include <KUser>
-#include <KFilePlacesModel>
+#include <QtTestWidgets>
 
-#include "kurlnavigator.h"
-#include "kurlcombobox.h"
 #include "kiotesthelper.h" // createTestDirectory(), createTestSymlink()
+#include "kurlcombobox.h"
+#include "kurlnavigator.h"
 
 QTEST_MAIN(KUrlNavigatorTest)
 
@@ -305,19 +305,25 @@ void KUrlNavigatorTest::testButtonUrl_data()
     QTest::addColumn<int>("buttonIndex");
     QTest::addColumn<QUrl>("expectedButtonUrl");
 
-    QTest::newRow("localPathButtonIndex3") << QUrl::fromLocalFile(QStringLiteral("/home/foo")) << 3 << QUrl::fromLocalFile(QStringLiteral("/home/foo")); // out of range
+    QTest::newRow("localPathButtonIndex3") << QUrl::fromLocalFile(QStringLiteral("/home/foo")) << 3
+                                           << QUrl::fromLocalFile(QStringLiteral("/home/foo")); // out of range
     QTest::newRow("localPathButtonIndex2") << QUrl::fromLocalFile(QStringLiteral("/home/foo")) << 2 << QUrl::fromLocalFile(QStringLiteral("/home/foo"));
     QTest::newRow("localPathButtonIndex1") << QUrl::fromLocalFile(QStringLiteral("/home/foo")) << 1 << QUrl::fromLocalFile(QStringLiteral("/home"));
     QTest::newRow("localPathButtonIndex0") << QUrl::fromLocalFile(QStringLiteral("/home/foo")) << 0 << QUrl::fromLocalFile(QStringLiteral("/"));
 
-    QTest::newRow("networkPathButtonIndex1") << QUrl::fromUserInput(QStringLiteral("network:/konqi.local/share")) << 1 << QUrl::fromUserInput(QStringLiteral("network:/konqi.local"));
-    QTest::newRow("networkPathButtonIndex0") << QUrl::fromUserInput(QStringLiteral("network:/konqi.local/share")) << 0 << QUrl::fromUserInput(QStringLiteral("network:/"));
+    QTest::newRow("networkPathButtonIndex1") << QUrl::fromUserInput(QStringLiteral("network:/konqi.local/share")) << 1
+                                             << QUrl::fromUserInput(QStringLiteral("network:/konqi.local"));
+    QTest::newRow("networkPathButtonIndex0") << QUrl::fromUserInput(QStringLiteral("network:/konqi.local/share")) << 0
+                                             << QUrl::fromUserInput(QStringLiteral("network:/"));
 
-    QTest::newRow("ftpPathButtonIndex1") << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/home/foo")) << 1 << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/home"));
-    QTest::newRow("ftpPathButtonIndex0") << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/home/foo")) << 0 << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/"));
+    QTest::newRow("ftpPathButtonIndex1") << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/home/foo")) << 1
+                                         << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/home"));
+    QTest::newRow("ftpPathButtonIndex0") << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/home/foo")) << 0
+                                         << QUrl::fromUserInput(QStringLiteral("ftp://kde.org/"));
 
     // bug 354678
-    QTest::newRow("localPathWithPercentage") << QUrl::fromLocalFile(QStringLiteral("/home/foo %/test")) << 2 << QUrl::fromLocalFile(QStringLiteral("/home/foo %"));
+    QTest::newRow("localPathWithPercentage") << QUrl::fromLocalFile(QStringLiteral("/home/foo %/test")) << 2
+                                             << QUrl::fromLocalFile(QStringLiteral("/home/foo %"));
 }
 
 void KUrlNavigatorTest::testButtonUrl()
@@ -346,8 +352,8 @@ void KUrlNavigatorTest::testButtonText()
 
     QList<QPushButton *> buttons = navigator.findChildren<QPushButton *>();
     const auto it = std::find_if(buttons.cbegin(), buttons.cend(), [](QPushButton *button) {
-            return button->text() == QLatin1String("&Here");
-            });
+        return button->text() == QLatin1String("&Here");
+    });
     QVERIFY(it != buttons.cend());
     QCOMPARE((*it)->property("plainText").toString(), QStringLiteral("Here"));
 }
