@@ -36,19 +36,7 @@ struct KCoreDirListerCacheDirectoryData;
 class KCoreDirListerPrivate
 {
 public:
-    explicit KCoreDirListerPrivate(KCoreDirLister *qq)
-        : q(qq)
-    {
-        complete = false;
-
-        autoUpdate = false;
-
-        delayedMimeTypes = false;
-
-        rootFileItem = KFileItem();
-
-        hasPendingChanges = false;
-    }
+    explicit KCoreDirListerPrivate(KCoreDirLister *qq);
 
     void emitCachedItems(const QUrl &, bool, bool);
     void slotInfoMessage(KJob *, const QString &);
@@ -103,13 +91,12 @@ public:
     // toplevel URL
     QUrl url;
 
-    bool complete : 1;
-
-    bool autoUpdate : 1;
-
-    bool delayedMimeTypes : 1;
-
-    bool hasPendingChanges : 1; // i.e. settings != oldSettings
+    bool complete = false;
+    bool autoUpdate = false;
+    bool delayedMimeTypes = false;
+    bool hasPendingChanges = false; // i.e. settings != oldSettings
+    bool m_autoErrorHandling = true;
+    bool requestMimeTypeWhileListing = false;
 
     struct JobData {
         long unsigned int percent, speed;
@@ -449,8 +436,6 @@ private:
         QSet<QString> listedFiles;
     };
 
-    // static const unsigned short MAX_JOBS_PER_LISTER;
-
     QMap<KIO::ListJob *, KIO::UDSEntryList> runningListJobs;
 
     // an item is a complete directory
@@ -504,8 +489,6 @@ struct KCoreDirListerCacheDirectoryData {
     void moveListersWithoutCachedItemsJob(const QUrl &url);
 };
 
-// const unsigned short KCoreDirListerCache::MAX_JOBS_PER_LISTER = 5;
-
 // This job tells KCoreDirListerCache to emit cached items asynchronously from listDir()
 // to give the KCoreDirLister user enough time for connecting to its signals, and so
 // that KCoreDirListerCache behaves just like when a real KIO::Job is used: nothing
