@@ -8,28 +8,28 @@
 #include "dropjob.h"
 
 #include "job_p.h"
-#include "pastejob.h"
-#include "pastejob_p.h"
 #include "jobuidelegate.h"
 #include "jobuidelegateextension.h"
 #include "kio_widgets_debug.h"
+#include "pastejob.h"
+#include "pastejob_p.h"
 
 #include <KConfigGroup>
 #include <KCoreDirLister>
 #include <KDesktopFile>
+#include <KFileItem>
+#include <KFileItemListProperties>
 #include <KIO/ApplicationLauncherJob>
 #include <KIO/CopyJob>
 #include <KIO/DndPopupMenuPlugin>
 #include <KIO/FileUndoManager>
-#include <KFileItem>
-#include <KFileItemListProperties>
 #include <KJobWidgets>
 #include <KLocalizedString>
-#include <KPluginMetaData>
 #include <KPluginLoader>
+#include <KPluginMetaData>
 #include <KProtocolManager>
-#include <KUrlMimeData>
 #include <KService>
+#include <KUrlMimeData>
 
 #include <QDropEvent>
 #include <QFileInfo>
@@ -42,7 +42,8 @@ using namespace KIO;
 
 Q_DECLARE_METATYPE(Qt::DropAction)
 
-namespace KIO {
+namespace KIO
+{
 class DropMenu;
 }
 
@@ -68,18 +69,17 @@ class KIO::DropJobPrivate : public KIO::JobPrivate
 {
 public:
     DropJobPrivate(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags)
-        : JobPrivate(),
-          // Extract everything from the dropevent, since it will be deleted before the job starts
-          m_mimeData(dropEvent->mimeData()),
-          m_urls(KUrlMimeData::urlsFromMimeData(m_mimeData, KUrlMimeData::PreferLocalUrls, &m_metaData)),
-          m_dropAction(dropEvent->dropAction()),
-          m_relativePos(dropEvent->pos()),
-          m_keyboardModifiers(dropEvent->keyboardModifiers()),
-          m_destUrl(destUrl),
-          m_destItem(KCoreDirLister::cachedItemForUrl(destUrl)),
-          m_flags(flags),
-          m_dropjobFlags(dropjobFlags),
-          m_triggered(false)
+        : JobPrivate()
+        , m_mimeData(dropEvent->mimeData()) // Extract everything from the dropevent, since it will be deleted before the job starts
+        , m_urls(KUrlMimeData::urlsFromMimeData(m_mimeData, KUrlMimeData::PreferLocalUrls, &m_metaData))
+        , m_dropAction(dropEvent->dropAction())
+        , m_relativePos(dropEvent->pos())
+        , m_keyboardModifiers(dropEvent->keyboardModifiers())
+        , m_destUrl(destUrl)
+        , m_destItem(KCoreDirLister::cachedItemForUrl(destUrl))
+        , m_flags(flags)
+        , m_dropjobFlags(dropjobFlags)
+        , m_triggered(false)
     {
         // Check for the drop of a bookmark -> we want a Link action
         if (m_mimeData->hasFormat(QStringLiteral("application/x-xbel"))) {
@@ -119,9 +119,9 @@ public:
         return true;
     }
     void handleCopyToDirectory();
+    void slotDropActionDetermined(int error);
     void handleDropToDesktopFile();
     void handleDropToExecutable();
-    int determineDropAction();
     void fillPopupMenu(KIO::DropMenu *popup);
     void addPluginActions(KIO::DropMenu *popup, const KFileItemListProperties &itemProps);
     void doCopyToDirectory();
@@ -138,7 +138,7 @@ public:
     const DropJobFlags m_dropjobFlags;
     QList<QAction *> m_appActions;
     QList<QAction *> m_pluginActions;
-    bool m_triggered;  // Tracks whether an action has been triggered in the popup menu.
+    bool m_triggered; // Tracks whether an action has been triggered in the popup menu.
     QSet<KIO::DropMenu *> m_menus;
 
     Q_DECLARE_PUBLIC(DropJob)
@@ -156,12 +156,11 @@ public:
         // the popup
         return job;
     }
-
 };
 
 DropMenu::DropMenu(QWidget *parent)
-    : QMenu(parent),
-      m_extraActionsSeparator(nullptr)
+    : QMenu(parent)
+    , m_extraActionsSeparator(nullptr)
 {
     m_cancelAction = new QAction(i18n("C&ancel") + QLatin1Char('\t') + QKeySequence(Qt::Key_Escape).toString(QKeySequence::NativeText), this);
     m_cancelAction->setIcon(QIcon::fromTheme(QStringLiteral("process-stop")));
@@ -243,73 +242,6 @@ void DropJobPrivate::slotStart()
     }
 }
 
-// Input: m_dropAction as set by Qt at the time of the drop event
-// Output: m_dropAction possibly modified
-// Returns a KIO error code, in case of error.
-int DropJobPrivate::determineDropAction()
-{
-    Q_Q(DropJob);
-
-    if (!KProtocolManager::supportsWriting(m_destUrl)) {
-        return KIO::ERR_CANNOT_WRITE;
-    }
-
-    if (!m_destItem.isNull() && !m_destItem.isWritable() && (m_flags & KIO::NoPrivilegeExecution)) {
-            return KIO::ERR_WRITE_ACCESS_DENIED;
-    }
-
-    bool allItemsAreFromTrash = true;
-    bool containsTrashRoot = false;
-    for (const QUrl &url : m_urls) {
-        const bool local = url.isLocalFile();
-        if (!local /*optimization*/ && url.scheme() == QLatin1String("trash")) {
-            if (url.path().isEmpty() || url.path() == QLatin1String("/")) {
-                containsTrashRoot = true;
-            }
-        } else {
-            allItemsAreFromTrash = false;
-        }
-        if (url.matches(m_destUrl, QUrl::StripTrailingSlash)) {
-            return KIO::ERR_DROP_ON_ITSELF;
-        }
-    }
-
-    const bool trashing = m_destUrl.scheme() == QLatin1String("trash");
-    if (trashing) {
-        if (allItemsAreFromTrash) {
-            qCDebug(KIO_WIDGETS) << "Dropping items from trash to trash";
-            return KIO::ERR_DROP_ON_ITSELF;
-        }
-        m_dropAction = Qt::MoveAction;
-        if (!q->uiDelegateExtension()->askDeleteConfirmation(m_urls, KIO::JobUiDelegate::Trash, KIO::JobUiDelegate::DefaultConfirmation)) {
-            return KIO::ERR_USER_CANCELED;
-        }
-        return KJob::NoError; // ok
-    }
-    const bool implicitCopy = m_destUrl.scheme() == QLatin1String("stash");
-    if (implicitCopy) {
-        m_dropAction = Qt::CopyAction;
-        return KJob::NoError; // ok
-    }
-    if (containsTrashRoot) {
-        // Dropping a link to the trash: don't move the full contents, just make a link (#319660)
-        m_dropAction = Qt::LinkAction;
-        return KJob::NoError; // ok
-    }
-    if (allItemsAreFromTrash) {
-        // No point in asking copy/move/link when using dragging from the trash, just move the file out.
-        m_dropAction = Qt::MoveAction;
-        return KJob::NoError; // ok
-    }
-    if (m_keyboardModifiers & (Qt::ControlModifier | Qt::ShiftModifier | Qt::AltModifier)) {
-        // Qt determined m_dropAction from the modifiers already
-        return KJob::NoError; // ok
-    }
-
-    // We need to ask the user with a popup menu. Let the caller know.
-    return KIO::ERR_UNKNOWN;
-}
-
 void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
 {
     Q_Q(DropJob);
@@ -324,7 +256,7 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
     }
     const KFileItemListProperties itemProps(fileItems);
 
-    emit q->popupMenuAboutToShow(itemProps);
+    Q_EMIT q->popupMenuAboutToShow(itemProps);
 
     const bool sReading = itemProps.supportsReading();
     const bool sDeleting = itemProps.supportsDeleting();
@@ -333,17 +265,17 @@ void DropJobPrivate::fillPopupMenu(KIO::DropMenu *popup)
     const int separatorLength = QCoreApplication::translate("QShortcut", "+").size();
     QString seq = QKeySequence(Qt::ShiftModifier).toString(QKeySequence::NativeText);
     seq.chop(separatorLength); // chop superfluous '+'
-    QAction* popupMoveAction = new QAction(i18n("&Move Here") + QLatin1Char('\t') + seq, popup);
+    QAction *popupMoveAction = new QAction(i18n("&Move Here") + QLatin1Char('\t') + seq, popup);
     popupMoveAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-move"), QIcon::fromTheme(QStringLiteral("go-jump"))));
     popupMoveAction->setData(QVariant::fromValue(Qt::MoveAction));
     seq = QKeySequence(Qt::ControlModifier).toString(QKeySequence::NativeText);
     seq.chop(separatorLength);
-    QAction* popupCopyAction = new QAction(i18n("&Copy Here") + QLatin1Char('\t') + seq, popup);
+    QAction *popupCopyAction = new QAction(i18n("&Copy Here") + QLatin1Char('\t') + seq, popup);
     popupCopyAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
     popupCopyAction->setData(QVariant::fromValue(Qt::CopyAction));
     seq = QKeySequence(Qt::ControlModifier | Qt::ShiftModifier).toString(QKeySequence::NativeText);
     seq.chop(separatorLength);
-    QAction* popupLinkAction = new QAction(i18n("&Link Here") + QLatin1Char('\t') + seq, popup);
+    QAction *popupLinkAction = new QAction(i18n("&Link Here") + QLatin1Char('\t') + seq, popup);
     popupLinkAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-link")));
     popupLinkAction->setData(QVariant::fromValue(Qt::LinkAction));
 
@@ -399,7 +331,6 @@ void DropJob::setApplicationActions(const QList<QAction *> &actions)
     for (KIO::DropMenu *menu : qAsConst(d->m_menus)) {
         menu->addExtraActions(d->m_appActions, d->m_pluginActions);
     }
-
 }
 
 void DropJob::showMenu(const QPoint &p, QAction *atAction)
@@ -450,47 +381,139 @@ void DropJobPrivate::handleCopyToDirectory()
 {
     Q_Q(DropJob);
 
-    if (int error = determineDropAction()) {
-        if (error == KIO::ERR_UNKNOWN) {
-            auto window = KJobWidgets::window(q);
-            KIO::DropMenu *menu = new KIO::DropMenu(window);
-            QObject::connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
+    // Process m_dropAction as set by Qt at the time of the drop event
+    if (!KProtocolManager::supportsWriting(m_destUrl)) {
+        slotDropActionDetermined(KIO::ERR_CANNOT_WRITE);
+        return;
+    }
 
-            // If the user clicks outside the menu, it will be destroyed without emitting the triggered signal.
-            QObject::connect(menu, &QMenu::aboutToHide, q, [this]() { slotAboutToHide(); });
+    if (!m_destItem.isNull() && !m_destItem.isWritable() && (m_flags & KIO::NoPrivilegeExecution)) {
+        slotDropActionDetermined(KIO::ERR_WRITE_ACCESS_DENIED);
+        return;
+    }
 
-            fillPopupMenu(menu);
-            QObject::connect(menu, &QMenu::triggered, q, [this](QAction* action) {
-                m_triggered = true;
-                slotTriggered(action);
-            });
-
-            if (!(m_dropjobFlags & KIO::ShowMenuManually)) {
-                menu->popup(window ? window->mapToGlobal(m_relativePos) : QCursor::pos());
+    bool allItemsAreFromTrash = true;
+    bool containsTrashRoot = false;
+    for (const QUrl &url : m_urls) {
+        const bool local = url.isLocalFile();
+        if (!local /*optimization*/ && url.scheme() == QLatin1String("trash")) {
+            if (url.path().isEmpty() || url.path() == QLatin1String("/")) {
+                containsTrashRoot = true;
             }
-            m_menus.insert(menu);
-            QObject::connect(menu, &QObject::destroyed, q, [this, menu]() {
-                m_menus.remove(menu);
-            });
         } else {
-            q->setError(error);
-            q->emitResult();
+            allItemsAreFromTrash = false;
         }
-    } else {
+        if (url.matches(m_destUrl, QUrl::StripTrailingSlash)) {
+            slotDropActionDetermined(KIO::ERR_DROP_ON_ITSELF);
+            return;
+        }
+    }
+
+    const bool trashing = m_destUrl.scheme() == QLatin1String("trash");
+    if (trashing) {
+        if (allItemsAreFromTrash) {
+            qCDebug(KIO_WIDGETS) << "Dropping items from trash to trash";
+            slotDropActionDetermined(KIO::ERR_DROP_ON_ITSELF);
+            return;
+        }
+        m_dropAction = Qt::MoveAction;
+
+        auto *askUserInterface = KIO::delegateExtension<AskUserActionInterface *>(q);
+
+        // No UI Delegate set for this job, or a delegate that doesn't implement
+        // AskUserActionInterface, then just proceed with the job without asking.
+        // This is useful for non-interactive usage, (which doesn't actually apply
+        // here as a DropJob is always interactive), but this is useful for unittests,
+        // which are typically non-interactive.
+        if (!askUserInterface) {
+            slotDropActionDetermined(KJob::NoError);
+            return;
+        }
+
+        QObject::connect(askUserInterface, &KIO::AskUserActionInterface::askUserDeleteResult, q, [this](bool allowDelete) {
+            if (allowDelete) {
+                slotDropActionDetermined(KJob::NoError);
+            } else {
+                slotDropActionDetermined(KIO::ERR_USER_CANCELED);
+            }
+        });
+
+        askUserInterface->askUserDelete(m_urls, KIO::AskUserActionInterface::Trash, KIO::AskUserActionInterface::DefaultConfirmation, KJobWidgets::window(q));
+        return;
+    }
+
+    // If we can't determine the action below, we use ERR::UNKNOWN as we need to ask
+    // the user via a popup menu.
+    int err = KIO::ERR_UNKNOWN;
+    const bool implicitCopy = m_destUrl.scheme() == QLatin1String("stash");
+    if (implicitCopy) {
+        m_dropAction = Qt::CopyAction;
+        err = KJob::NoError; // Ok
+    } else if (containsTrashRoot) {
+        // Dropping a link to the trash: don't move the full contents, just make a link (#319660)
+        m_dropAction = Qt::LinkAction;
+        err = KJob::NoError; // Ok
+    } else if (allItemsAreFromTrash) {
+        // No point in asking copy/move/link when using dragging from the trash, just move the file out.
+        m_dropAction = Qt::MoveAction;
+        err = KJob::NoError; // Ok
+    } else if (m_keyboardModifiers & (Qt::ControlModifier | Qt::ShiftModifier | Qt::AltModifier)) {
+        // Qt determined m_dropAction from the modifiers already
+        err = KJob::NoError; // Ok
+    }
+    slotDropActionDetermined(err);
+}
+
+void DropJobPrivate::slotDropActionDetermined(int error)
+{
+    Q_Q(DropJob);
+
+    if (error == KJob::NoError) {
         doCopyToDirectory();
+        return;
+    }
+
+    // There was an error, handle it
+    if (error == KIO::ERR_UNKNOWN) {
+        auto *window = KJobWidgets::window(q);
+        KIO::DropMenu *menu = new KIO::DropMenu(window);
+        QObject::connect(menu, &QMenu::aboutToHide, menu, &QObject::deleteLater);
+
+        // If the user clicks outside the menu, it will be destroyed without emitting the triggered signal.
+        QObject::connect(menu, &QMenu::aboutToHide, q, [this]() {
+            slotAboutToHide();
+        });
+
+        fillPopupMenu(menu);
+        QObject::connect(menu, &QMenu::triggered, q, [this](QAction *action) {
+            m_triggered = true;
+            slotTriggered(action);
+        });
+
+        if (!(m_dropjobFlags & KIO::ShowMenuManually)) {
+            menu->popup(window ? window->mapToGlobal(m_relativePos) : QCursor::pos());
+        }
+        m_menus.insert(menu);
+        QObject::connect(menu, &QObject::destroyed, q, [this, menu]() {
+            m_menus.remove(menu);
+        });
+    } else {
+        q->setError(error);
+        q->emitResult();
     }
 }
 
 void DropJobPrivate::doCopyToDirectory()
 {
     Q_Q(DropJob);
-    KIO::CopyJob * job = nullptr;
+    KIO::CopyJob *job = nullptr;
     switch (m_dropAction) {
     case Qt::MoveAction:
         job = KIO::move(m_urls, m_destUrl, m_flags);
-        KIO::FileUndoManager::self()->recordJob(
-            m_destUrl.scheme() == QLatin1String("trash") ? KIO::FileUndoManager::Trash : KIO::FileUndoManager::Move,
-            m_urls, m_destUrl, job);
+        KIO::FileUndoManager::self()->recordJob(m_destUrl.scheme() == QLatin1String("trash") ? KIO::FileUndoManager::Trash : KIO::FileUndoManager::Move,
+                                                m_urls,
+                                                m_destUrl,
+                                                job);
         break;
     case Qt::CopyAction:
         job = KIO::copy(m_urls, m_destUrl, m_flags);
@@ -507,18 +530,17 @@ void DropJobPrivate::doCopyToDirectory()
         return;
     }
     Q_ASSERT(job);
-    job->setUiDelegate(q->uiDelegate());
     job->setParentJob(q);
     job->setMetaData(m_metaData);
-    QObject::connect(job, &KIO::CopyJob::copyingDone, q, [q](KIO::Job*, const QUrl &, const QUrl &to) {
-            emit q->itemCreated(to);
+    QObject::connect(job, &KIO::CopyJob::copyingDone, q, [q](KIO::Job *, const QUrl &, const QUrl &to) {
+        Q_EMIT q->itemCreated(to);
     });
-    QObject::connect(job, &KIO::CopyJob::copyingLinkDone, q, [q](KIO::Job*, const QUrl&, const QString&, const QUrl &to) {
-            emit q->itemCreated(to);
+    QObject::connect(job, &KIO::CopyJob::copyingLinkDone, q, [q](KIO::Job *, const QUrl &, const QString &, const QUrl &to) {
+        Q_EMIT q->itemCreated(to);
     });
     q->addSubjob(job);
 
-    emit q->copyJobStarted(job);
+    Q_EMIT q->copyJobStarted(job);
 }
 
 void DropJobPrivate::handleDropToDesktopFile()
@@ -531,10 +553,10 @@ void DropJobPrivate::handleDropToDesktopFile()
     if (desktopFile.hasApplicationType()) {
         // Drop to application -> start app with urls as argument
         KService::Ptr service(new KService(destFile));
-        KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(service);
+        // Can't use setParentJob() because ApplicationLauncherJob isn't a KIO::Job,
+        // intead pass q as parent so that KIO::delegateExtension() can find a delegate
+        KIO::ApplicationLauncherJob *job = new KIO::ApplicationLauncherJob(service, q);
         job->setUrls(m_urls);
-        job->setUiDelegate(q->uiDelegate());
-        job->start();
         QObject::connect(job, &KJob::result, q, [=]() {
             if (job->error()) {
                 q->setError(KIO::ERR_CANNOT_LAUNCH_PROCESS);
@@ -542,6 +564,7 @@ void DropJobPrivate::handleDropToDesktopFile()
             }
             q->emitResult();
         });
+        job->start();
     } else if (desktopFile.hasLinkType() && desktopGroup.hasKey(urlKey)) {
         // Drop to link -> adjust destination directory
         m_destUrl = QUrl::fromUserInput(desktopGroup.readPathEntry(urlKey, QString()));
@@ -581,15 +604,15 @@ void DropJob::slotResult(KJob *job)
     emitResult();
 }
 
-DropJob * KIO::drop(const QDropEvent *dropEvent, const QUrl &destUrl, JobFlags flags)
+DropJob *KIO::drop(const QDropEvent *dropEvent, const QUrl &destUrl, JobFlags flags)
 {
     return DropJobPrivate::newJob(dropEvent, destUrl, KIO::DropJobDefaultFlags, flags);
 }
 
-DropJob * KIO::drop(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags)
+DropJob *KIO::drop(const QDropEvent *dropEvent, const QUrl &destUrl, DropJobFlags dropjobFlags, JobFlags flags)
 {
     return DropJobPrivate::newJob(dropEvent, destUrl, dropjobFlags, flags);
 }
 
-#include "moc_dropjob.cpp"
 #include "dropjob.moc"
+#include "moc_dropjob.cpp"
