@@ -1,23 +1,27 @@
 /*
     This file is part of the KDE project
     SPDX-FileCopyrightText: 1998-2009 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2021 Alexander Lohnau <alexander.lohnau@gmx.de>
 
     SPDX-License-Identifier: LGPL-2.0-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */
 
 #include "kfileitemactions.h"
 #include "kfileitemactions_p.h"
-#include <KMimeTypeTrader>
-#include <kdesktopfileactions.h>
-#include <KLocalizedString>
-#include <kurlauthorized.h>
+#include <KAbstractFileItemActionPlugin>
+#include <KApplicationTrader>
+#include <KAuthorized>
 #include <KConfigGroup>
 #include <KDesktopFile>
-#include <KServiceTypeTrader>
-#include <KAbstractFileItemActionPlugin>
-#include <KPluginMetaData>
 #include <KIO/ApplicationLauncherJob>
 #include <KIO/JobUiDelegate>
+#include <KLocalizedString>
+#include <KMimeTypeTrader>
+#include <KPluginMetaData>
+#include <KServiceTypeTrader>
+#include <kapplicationtrader.h>
+#include <kdesktopfileactions.h>
+#include <kurlauthorized.h>
 
 #include <QFile>
 #include <QMenu>
@@ -28,14 +32,14 @@
 #include <QDBusConnectionInterface>
 #include <QDBusInterface>
 #include <QDBusMessage>
-#include <kio_widgets_debug.h>
 #include <algorithm>
+#include <kio_widgets_debug.h>
 
 static bool KIOSKAuthorizedAction(const KConfigGroup &cfg)
 {
     const QStringList list = cfg.readEntry("X-KDE-AuthorizeAction", QStringList());
     return std::all_of(list.constBegin(), list.constEnd(), [](const QString &action) {
-         return KAuthorized::authorize(action.trimmed());
+        return KAuthorized::authorize(action.trimmed());
     });
 }
 
@@ -46,8 +50,7 @@ static bool mimeTypeListContains(const QStringList &list, const KFileItem &item)
         if (i == itemMimeType || i == QLatin1String("all/all")) {
             return true;
         }
-        if (item.isFile() && (i == QLatin1String("allfiles") ||
-            i == QLatin1String("all/allfiles") || i == QLatin1String("application/octet-stream"))) {
+        if (item.isFile() && (i == QLatin1String("allfiles") || i == QLatin1String("all/allfiles") || i == QLatin1String("application/octet-stream"))) {
             return true;
         }
 
@@ -57,7 +60,7 @@ static bool mimeTypeListContains(const QStringList &list, const KFileItem &item)
 
         const int iSlashPos = i.indexOf(QLatin1Char(QLatin1Char('/')));
         Q_ASSERT(iSlashPos > 0);
-        const QStringRef iSubType = i.midRef(iSlashPos+1);
+        const QStringRef iSubType = i.midRef(iSlashPos + 1);
 
         if (iSubType == QLatin1String("*")) {
             const int itemSlashPos = itemMimeType.indexOf(QLatin1Char('/'));
@@ -71,8 +74,6 @@ static bool mimeTypeListContains(const QStringList &list, const KFileItem &item)
     });
 }
 
-
-
 // This helper class stores the .desktop-file actions and the servicemenus
 // in order to support X-KDE-Priority and X-KDE-Submenu.
 namespace KIO
@@ -82,9 +83,17 @@ class PopupServices
 public:
     ServiceList &selectList(const QString &priority, const QString &submenuName);
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
     ServiceList builtin;
-    ServiceList user, userToplevel, userPriority;
-    QMap<QString, ServiceList> userSubmenus, userToplevelSubmenus, userPrioritySubmenus;
+#endif
+
+    ServiceList user;
+    ServiceList userToplevel;
+    ServiceList userPriority;
+
+    QMap<QString, ServiceList> userSubmenus;
+    QMap<QString, ServiceList> userToplevelSubmenus;
+    QMap<QString, ServiceList> userPrioritySubmenus;
 };
 
 ServiceList &PopupServices::selectList(const QString &priority, const QString &submenuName)
@@ -111,32 +120,28 @@ ServiceList &PopupServices::selectList(const QString &priority, const QString &s
 ////
 
 KFileItemActionsPrivate::KFileItemActionsPrivate(KFileItemActions *qq)
-    : QObject(),
-      q(qq),
-      m_executeServiceActionGroup(static_cast<QWidget *>(nullptr)),
-      m_runApplicationActionGroup(static_cast<QWidget *>(nullptr)),
-      m_parentWidget(nullptr),
-      m_config(QStringLiteral("kservicemenurc"), KConfig::NoGlobals)
+    : QObject()
+    , q(qq)
+    , m_executeServiceActionGroup(static_cast<QWidget *>(nullptr))
+    , m_runApplicationActionGroup(static_cast<QWidget *>(nullptr))
+    , m_parentWidget(nullptr)
+    , m_config(QStringLiteral("kservicemenurc"), KConfig::NoGlobals)
 {
-    QObject::connect(&m_executeServiceActionGroup, &QActionGroup::triggered,
-                     this, &KFileItemActionsPrivate::slotExecuteService);
-    QObject::connect(&m_runApplicationActionGroup, &QActionGroup::triggered,
-                     this, &KFileItemActionsPrivate::slotRunApplication);
+    QObject::connect(&m_executeServiceActionGroup, &QActionGroup::triggered, this, &KFileItemActionsPrivate::slotExecuteService);
+    QObject::connect(&m_runApplicationActionGroup, &QActionGroup::triggered, this, &KFileItemActionsPrivate::slotRunApplication);
 }
 
 KFileItemActionsPrivate::~KFileItemActionsPrivate()
 {
 }
 
-int KFileItemActionsPrivate::insertServicesSubmenus(const QMap<QString, ServiceList> &submenus,
-        QMenu *menu,
-        bool isBuiltin)
+int KFileItemActionsPrivate::insertServicesSubmenus(const QMap<QString, ServiceList> &submenus, QMenu *menu, bool isBuiltin)
 {
     int count = 0;
     QMap<QString, ServiceList>::ConstIterator it;
     for (it = submenus.begin(); it != submenus.end(); ++it) {
         if (it.value().isEmpty()) {
-            //avoid empty sub-menus
+            // avoid empty sub-menus
             continue;
         }
 
@@ -151,9 +156,7 @@ int KFileItemActionsPrivate::insertServicesSubmenus(const QMap<QString, ServiceL
     return count;
 }
 
-int KFileItemActionsPrivate::insertServices(const ServiceList &list,
-        QMenu *menu,
-        bool isBuiltin)
+int KFileItemActionsPrivate::insertServices(const ServiceList &list, QMenu *menu, bool isBuiltin)
 {
     int count = 0;
     for (const KServiceAction &serviceAction : list) {
@@ -194,7 +197,8 @@ void KFileItemActionsPrivate::slotExecuteService(QAction *act)
 }
 
 KFileItemActions::KFileItemActions(QObject *parent)
-    : QObject(parent), d(new KFileItemActionsPrivate(this))
+    : QObject(parent)
+    , d(new KFileItemActionsPrivate(this))
 {
 }
 
@@ -216,20 +220,21 @@ void KFileItemActions::setItemListProperties(const KFileItemListProperties &item
     }
 }
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 79)
 int KFileItemActions::addServiceActionsTo(QMenu *mainMenu)
 {
     return d->addServiceActionsTo(mainMenu, {}, {}).first;
 }
+#endif
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 79)
 int KFileItemActions::addPluginActionsTo(QMenu *mainMenu)
 {
     return d->addPluginActionsTo(mainMenu, mainMenu, {});
 }
+#endif
 
-void KFileItemActions::addActionsTo(QMenu *menu,
-                                    MenuActionSources sources,
-                                    const QList<QAction *> &additionalActions,
-                                    const QStringList &excludeList)
+void KFileItemActions::addActionsTo(QMenu *menu, MenuActionSources sources, const QList<QAction *> &additionalActions, const QStringList &excludeList)
 {
     QMenu *actionsMenu = menu;
     if (sources & MenuActionSource::Services) {
@@ -245,186 +250,72 @@ void KFileItemActions::addActionsTo(QMenu *menu,
     }
 }
 
+// static
+KService::List KFileItemActions::associatedApplications(const QStringList &mimeTypeList)
+{
+    return KFileItemActionsPrivate::associatedApplications(mimeTypeList, QString(), QStringList{});
+}
+
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 83)
 // static
 KService::List KFileItemActions::associatedApplications(const QStringList &mimeTypeList, const QString &traderConstraint)
 {
-    if (!KAuthorized::authorizeAction(QStringLiteral("openwith")) || mimeTypeList.isEmpty()) {
-        return KService::List();
-    }
-
-    const KService::List firstOffers = KMimeTypeTrader::self()->query(mimeTypeList.first(), QStringLiteral("Application"), traderConstraint);
-
-    QList<KFileItemActionsPrivate::ServiceRank> rankings;
-    QStringList serviceList;
-
-    // This section does two things.  First, it determines which services are common to all the given MIME types.
-    // Second, it ranks them based on their preference level in the associated applications list.
-    // The more often a service appear near the front of the list, the LOWER its score.
-
-    rankings.reserve(firstOffers.count());
-    serviceList.reserve(firstOffers.count());
-    for (int i = 0; i < firstOffers.count(); ++i) {
-        KFileItemActionsPrivate::ServiceRank tempRank;
-        tempRank.service = firstOffers[i];
-        tempRank.score = i;
-        rankings << tempRank;
-        serviceList << tempRank.service->storageId();
-    }
-
-    for (int j = 1; j < mimeTypeList.count(); ++j) {
-        QStringList subservice; // list of services that support this MIME type
-        const KService::List offers = KMimeTypeTrader::self()->query(mimeTypeList[j], QStringLiteral("Application"), traderConstraint);
-        subservice.reserve(offers.count());
-        for (int i = 0; i != offers.count(); ++i) {
-            const QString serviceId = offers[i]->storageId();
-            subservice << serviceId;
-            const int idPos = serviceList.indexOf(serviceId);
-            if (idPos != -1) {
-                rankings[idPos].score += i;
-            } // else: we ignore the services that didn't support the previous MIME types
-        }
-
-        // Remove services which supported the previous MIME types but don't support this one
-        for (int i = 0; i < serviceList.count(); ++i) {
-            if (!subservice.contains(serviceList[i])) {
-                serviceList.removeAt(i);
-                rankings.removeAt(i);
-                --i;
-            }
-        }
-        // Nothing left -> there is no common application for these MIME types
-        if (rankings.isEmpty()) {
-            return KService::List();
-        }
-    }
-
-    std::sort(rankings.begin(), rankings.end(), KFileItemActionsPrivate::lessRank);
-
-    KService::List result;
-    result.reserve(rankings.size());
-    for (const KFileItemActionsPrivate::ServiceRank &tempRank : qAsConst(rankings)) {
-        result << tempRank.service;
-    }
-
-    return result;
+    return KFileItemActionsPrivate::associatedApplications(mimeTypeList, traderConstraint, QStringList{});
 }
 
-// KMimeTypeTrader::preferredService doesn't take a constraint
-static KService::Ptr preferredService(const QString &mimeType, const QString &constraint)
+#endif
+
+static KService::Ptr preferredService(const QString &mimeType, const QStringList &excludedDesktopEntryNames, const QString &constraint)
 {
-    const KService::List services = KMimeTypeTrader::self()->query(mimeType, QStringLiteral("Application"), constraint);
-    return !services.isEmpty() ? services.first() : KService::Ptr();
+    KService::List services;
+    if (constraint.isEmpty()) {
+        services = KApplicationTrader::queryByMimeType(mimeType, [&](const KService::Ptr &serv) {
+            return !excludedDesktopEntryNames.contains(serv->desktopEntryName());
+        });
+    }
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
+    else {
+        Q_ASSERT(excludedDesktopEntryNames.isEmpty());
+        // KMimeTypeTrader::preferredService doesn't take a constraint
+        QT_WARNING_PUSH
+        QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+        QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+        services = KMimeTypeTrader::self()->query(mimeType, QStringLiteral("Application"), constraint);
+        QT_WARNING_POP
+    }
+#endif
+    return services.isEmpty() ? KService::Ptr() : services.first();
 }
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
 void KFileItemActions::addOpenWithActionsTo(QMenu *topMenu, const QString &traderConstraint)
 {
-    insertOpenWithActionsTo(nullptr /* append actions */, topMenu, traderConstraint);
+    d->insertOpenWithActionsTo(nullptr, topMenu, QStringList(), traderConstraint);
 }
+#endif
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
 void KFileItemActions::insertOpenWithActionsTo(QAction *before, QMenu *topMenu, const QString &traderConstraint)
 {
-    if (!KAuthorized::authorizeAction(QStringLiteral("openwith"))) {
-        return;
-    }
+    d->insertOpenWithActionsTo(before, topMenu, QStringList(), traderConstraint);
+}
+#endif
 
-    d->m_traderConstraint = traderConstraint;
-    KService::List offers = associatedApplications(d->m_mimeTypeList, traderConstraint);
-
-    //// Ok, we have everything, now insert
-
-    const KFileItemList items = d->m_props.items();
-    const KFileItem &firstItem = items.first();
-    const bool isLocal = firstItem.url().isLocalFile();
-    const bool isDir = d->m_props.isDirectory();
-    // "Open With..." for folders is really not very useful, especially for remote folders.
-    // (media:/something, or trash:/, or ftp://...).
-    // Don't show "open with" actions for remote dirs only
-    if (isDir && !isLocal) {
-        return;
-    }
-
-    const QStringList serviceIdList = d->listPreferredServiceIds(d->m_mimeTypeList, traderConstraint);
-
-    // When selecting files with multiple MIME types, offer either "open with <app for all>"
-    // or a generic <open> (if there are any apps associated).
-    if (d->m_mimeTypeList.count() > 1
-            && !serviceIdList.isEmpty()
-            && !(serviceIdList.count() == 1 && serviceIdList.first().isEmpty())) { // empty means "no apps associated"
-
-        QAction *runAct = new QAction(this);
-        if (serviceIdList.count() == 1) {
-            const KService::Ptr app = preferredService(d->m_mimeTypeList.first(), traderConstraint);
-            runAct->setText(i18n("&Open with %1", app->name()));
-            runAct->setIcon(QIcon::fromTheme(app->icon()));
-
-            // Remove that app from the offers list (#242731)
-            for (int i = 0; i < offers.count(); ++i) {
-                if (offers[i]->storageId() == app->storageId()) {
-                    offers.removeAt(i);
-                    break;
-                }
-            }
-        } else {
-            runAct->setText(i18n("&Open"));
-        }
-
-        QObject::connect(runAct, &QAction::triggered, d, &KFileItemActionsPrivate::slotRunPreferredApplications);
-        topMenu->insertAction(before, runAct);
-
-        d->m_traderConstraint = traderConstraint;
-        d->m_fileOpenList = d->m_props.items();
-    }
-
-    QAction *openWithAct = new QAction(this);
-    openWithAct->setText(i18nc("@title:menu", "&Open With..."));
-    openWithAct->setObjectName(QStringLiteral("openwith_browse")); // For the unittest
-    QObject::connect(openWithAct, &QAction::triggered, d, &KFileItemActionsPrivate::slotOpenWithDialog);
-
-    if (!offers.isEmpty()) {
-        // Show the top app inline for files, but not folders
-        if (!isDir) {
-            QAction *act = d->createAppAction(offers.takeFirst(), true);
-            topMenu->insertAction(before, act);
-        }
-
-        // If there are still more apps, show them in a sub-menu
-        if (!offers.isEmpty()) { // submenu 'open with'
-            QMenu *subMenu = new QMenu(i18nc("@title:menu", "&Open With"), topMenu);
-            subMenu->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
-            subMenu->menuAction()->setObjectName(QStringLiteral("openWith_submenu")); // For the unittest
-            // Add other apps to the sub-menu
-            for (const KServicePtr &service : qAsConst(offers)) {
-                QAction *act = d->createAppAction(service, false);
-                subMenu->addAction(act);
-            }
-
-            subMenu->addSeparator();
-
-            openWithAct->setText(i18nc("@action:inmenu Open With", "&Other Application..."));
-            subMenu->addAction(openWithAct);
-
-            topMenu->insertMenu(before, subMenu);
-            topMenu->insertSeparator(before);
-        } else { // No other apps
-            topMenu->insertAction(before, openWithAct);
-        }
-    } else { // no app offers -> Open With...
-        openWithAct->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
-        openWithAct->setObjectName(QStringLiteral("openwith")); // For the unittest
-        topMenu->insertAction(before, openWithAct);
-    }
+void KFileItemActions::insertOpenWithActionsTo(QAction *before, QMenu *topMenu, const QStringList &excludedDesktopEntryNames)
+{
+    d->insertOpenWithActionsTo(before, topMenu, excludedDesktopEntryNames, QString());
 }
 
 void KFileItemActionsPrivate::slotRunPreferredApplications()
 {
     const KFileItemList fileItems = m_fileOpenList;
     const QStringList mimeTypeList = listMimeTypes(fileItems);
-    const QStringList serviceIdList = listPreferredServiceIds(mimeTypeList, m_traderConstraint);
+    const QStringList serviceIdList = listPreferredServiceIds(mimeTypeList, QStringList(), m_traderConstraint);
 
-    for (const QString& serviceId : serviceIdList) {
+    for (const QString &serviceId : serviceIdList) {
         KFileItemList serviceItems;
         for (const KFileItem &item : fileItems) {
-            const KService::Ptr serv = preferredService(item.mimetype(), m_traderConstraint);
+            const KService::Ptr serv = preferredService(item.mimetype(), QStringList(), m_traderConstraint);
             const QString preferredServiceId = serv ? serv->storageId() : QString();
             if (preferredServiceId == serviceId) {
                 serviceItems << item;
@@ -444,17 +335,26 @@ void KFileItemActionsPrivate::slotRunPreferredApplications()
     }
 }
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 83)
 void KFileItemActions::runPreferredApplications(const KFileItemList &fileOpenList, const QString &traderConstraint)
 {
     d->m_fileOpenList = fileOpenList;
     d->m_traderConstraint = traderConstraint;
     d->slotRunPreferredApplications();
 }
+#endif
+
+void KFileItemActions::runPreferredApplications(const KFileItemList &fileOpenList)
+{
+    d->m_fileOpenList = fileOpenList;
+    d->m_traderConstraint = QString();
+    d->slotRunPreferredApplications();
+}
 
 void KFileItemActionsPrivate::openWithByMime(const KFileItemList &fileItems)
 {
     const QStringList mimeTypeList = listMimeTypes(fileItems);
-    for (const QString& mimeType : mimeTypeList) {
+    for (const QString &mimeType : mimeTypeList) {
         KFileItemList mimeItems;
         for (const KFileItem &item : fileItems) {
             if (item.mimetype() == mimeType) {
@@ -483,7 +383,7 @@ void KFileItemActionsPrivate::slotRunApplication(QAction *act)
 void KFileItemActionsPrivate::slotOpenWithDialog()
 {
     // The item 'Other...' or 'Open With...' has been selected
-    emit q->openWithDialogAboutToBeShown();
+    Q_EMIT q->openWithDialogAboutToBeShown();
     auto *job = new KIO::ApplicationLauncherJob();
     job->setUrls(m_props.urlList());
     job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, m_parentWidget));
@@ -501,14 +401,14 @@ QStringList KFileItemActionsPrivate::listMimeTypes(const KFileItemList &items)
     return mimeTypeList;
 }
 
-QStringList KFileItemActionsPrivate::listPreferredServiceIds(const QStringList &mimeTypeList, const QString &traderConstraint)
+QStringList
+KFileItemActionsPrivate::listPreferredServiceIds(const QStringList &mimeTypeList, const QStringList &excludedDesktopEntryNames, const QString &traderConstraint)
 {
     QStringList serviceIdList;
     serviceIdList.reserve(mimeTypeList.size());
     for (const QString &mimeType : mimeTypeList) {
-        const KService::Ptr serv = preferredService(mimeType, traderConstraint);
-        const QString newOffer = serv ? serv->storageId() : QString();
-        serviceIdList << newOffer;
+        const KService::Ptr serv = preferredService(mimeType, excludedDesktopEntryNames, traderConstraint);
+        serviceIdList << (serv ? serv->storageId() : QString()); // empty string means mimetype has no associated apps
     }
     serviceIdList.removeDuplicates();
     return serviceIdList;
@@ -560,12 +460,10 @@ bool KFileItemActionsPrivate::shouldDisplayServiceMenu(const KConfigGroup &cfg,
             interface.truncate(pos);
         }
 
-        QDBusMessage reply = QDBusInterface(app, obj, interface).
-            call(method, QUrl::toStringList(urlList));
+        QDBusMessage reply = QDBusInterface(app, obj, interface).call(method, QUrl::toStringList(urlList));
         if (reply.arguments().count() < 1 || reply.arguments().at(0).type() != QVariant::Bool || !reply.arguments().at(0).toBool()) {
             return false;
         }
-
     }
 #endif
     if (cfg.hasKey("X-KDE-Protocol")) {
@@ -623,8 +521,7 @@ bool KFileItemActionsPrivate::checkTypesMatch(const KConfigGroup &cfg) const
 {
     // Like KService, we support ServiceTypes, X-KDE-ServiceTypes, and MimeType.
     const QStringList types = cfg.readEntry("ServiceTypes", QStringList())
-        << cfg.readEntry("X-KDE-ServiceTypes", QStringList())
-        << cfg.readXdgListEntry("MimeType");
+        << cfg.readEntry("X-KDE-ServiceTypes", QStringList()) << cfg.readXdgListEntry("MimeType");
 
     if (types.isEmpty()) {
         return false;
@@ -632,17 +529,12 @@ bool KFileItemActionsPrivate::checkTypesMatch(const KConfigGroup &cfg) const
 
     const QStringList excludeTypes = cfg.readEntry("ExcludeServiceTypes", QStringList());
     const KFileItemList items = m_props.items();
-    return std::all_of(items.constBegin(), items.constEnd(),
-                                [&types, &excludeTypes](const KFileItem &i)
-                                {
-                                    return mimeTypeListContains(types, i) && !mimeTypeListContains(excludeTypes, i);
-                                });
-
+    return std::all_of(items.constBegin(), items.constEnd(), [&types, &excludeTypes](const KFileItem &i) {
+        return mimeTypeListContains(types, i) && !mimeTypeListContains(excludeTypes, i);
+    });
 }
 
-QPair<int, QMenu *> KFileItemActionsPrivate::addServiceActionsTo(QMenu *mainMenu,
-    const QList<QAction *> &additionalActions,
-    const QStringList &excludeList)
+QPair<int, QMenu *> KFileItemActionsPrivate::addServiceActionsTo(QMenu *mainMenu, const QList<QAction *> &additionalActions, const QStringList &excludeList)
 {
     const KFileItemList items = m_props.items();
     const KFileItem &firstItem = items.first();
@@ -653,11 +545,14 @@ QPair<int, QMenu *> KFileItemActionsPrivate::addServiceActionsTo(QMenu *mainMenu
 
     KIO::PopupServices s;
 
+    // TODO KF6 remove mention of "builtin" (deprecated)
     // 1 - Look for builtin and user-defined services
     if (isSingleLocal && m_props.mimeType() == QLatin1String("application/x-desktop")) {
-        // get builtin services, like mount/unmount
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
+        // Get builtin services, like mount/unmount
+        s.builtin = KDesktopFileActions::builtinServices(QUrl::fromLocalFile(firstItem.localPath()));
+#endif
         const QString path = firstItem.localPath();
-        s.builtin = KDesktopFileActions::builtinServices(QUrl::fromLocalFile(path));
         const KDesktopFile desktopFile(path);
         const KConfigGroup cfg = desktopFile.desktopGroup();
         const QString priority = cfg.readEntry("X-KDE-Priority");
@@ -717,8 +612,7 @@ QPair<int, QMenu *> KFileItemActionsPrivate::addServiceActionsTo(QMenu *mainMenu
 
     QMenu *actionMenu = mainMenu;
     int userItemCount = 0;
-    if (s.user.count() + s.userSubmenus.count() +
-        s.userPriority.count() + s.userPrioritySubmenus.count() + additionalActions.count() > 3) {
+    if (s.user.count() + s.userSubmenus.count() + s.userPriority.count() + s.userPrioritySubmenus.count() + additionalActions.count() > 3) {
         // we have more than three items, so let's make a submenu
         actionMenu = new QMenu(i18nc("@title:menu", "&Actions"), mainMenu);
         actionMenu->setIcon(QIcon::fromTheme(QStringLiteral("view-more-symbolic")));
@@ -734,16 +628,18 @@ QPair<int, QMenu *> KFileItemActionsPrivate::addServiceActionsTo(QMenu *mainMenu
     userItemCount += insertServices(s.userPriority, actionMenu, false);
     userItemCount += insertServicesSubmenus(s.userSubmenus, actionMenu, false);
     userItemCount += insertServices(s.user, actionMenu, false);
+
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
     userItemCount += insertServices(s.builtin, mainMenu, true);
+#endif
+
     userItemCount += insertServicesSubmenus(s.userToplevelSubmenus, mainMenu, false);
     userItemCount += insertServices(s.userToplevel, mainMenu, false);
 
     return {userItemCount, actionMenu};
 }
 
-int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu,
-    QMenu *actionsMenu,
-    const QStringList &excludeList)
+int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu, QMenu *actionsMenu, const QStringList &excludeList)
 {
     QString commonMimeType = m_props.mimeType();
     if (commonMimeType.isEmpty() && m_props.isFile()) {
@@ -754,29 +650,9 @@ int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu,
     int itemCount = 0;
 
     const KConfigGroup showGroup = m_config.group("Show");
-    const KService::List fileItemPlugins = KMimeTypeTrader::self()->query(commonMimeType, QStringLiteral("KFileItemAction/Plugin"), QStringLiteral("exist Library"));
-    for(const auto &service : fileItemPlugins) {
-        if (!showGroup.readEntry(service->desktopEntryName(), true)) {
-            // The plugin has been disabled
-            continue;
-        }
-
-        auto *abstractPlugin = service->createInstance<KAbstractFileItemActionPlugin>();
-        if (abstractPlugin) {
-            abstractPlugin->setParent(mainMenu);
-            auto actions = abstractPlugin->actions(m_props, m_parentWidget);
-            itemCount += actions.count();
-            mainMenu->addActions(actions);
-            addedPlugins.append(service->desktopEntryName());
-        }
-    }
 
     const QMimeDatabase db;
-    const auto jsonPlugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kfileitemaction"), [&db, commonMimeType](const KPluginMetaData& metaData) {
-        if (!metaData.serviceTypes().contains(QLatin1String("KFileItemAction/Plugin"))) {
-            return false;
-        }
-
+    const auto jsonPlugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kfileitemaction"), [&db, commonMimeType](const KPluginMetaData &metaData) {
         auto mimeType = db.mimeTypeForName(commonMimeType);
         const QStringList list = metaData.mimeTypes();
         return std::any_of(list.constBegin(), list.constEnd(), [mimeType](const QString &supportedMimeType) {
@@ -791,18 +667,17 @@ int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu,
             continue;
         }
 
-        // The plugin also has a .desktop file and has already been added.
-        if (addedPlugins.contains(jsonMetadata.pluginId())) {
-            continue;
-        }
-
         KPluginFactory *factory = KPluginLoader(jsonMetadata.fileName()).factory();
         if (!factory) {
             continue;
         }
-        auto *abstractPlugin = factory->create<KAbstractFileItemActionPlugin>();
+        KAbstractFileItemActionPlugin *abstractPlugin = m_loadedPlugins.value(pluginId);
+        if (!abstractPlugin) {
+            abstractPlugin = factory->create<KAbstractFileItemActionPlugin>(this);
+            m_loadedPlugins.insert(pluginId, abstractPlugin);
+        }
         if (abstractPlugin) {
-            abstractPlugin->setParent(this);
+            connect(abstractPlugin, &KAbstractFileItemActionPlugin::error, q, &KFileItemActions::error);
             const QList<QAction *> actions = abstractPlugin->actions(m_props, m_parentWidget);
             itemCount += actions.count();
             const QString showInSubmenu = jsonMetadata.value(QStringLiteral("X-KDE-Show-In-Submenu"));
@@ -815,9 +690,239 @@ int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu,
         }
     }
 
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    const KService::List fileItemPlugins =
+        KMimeTypeTrader::self()->query(commonMimeType, QStringLiteral("KFileItemAction/Plugin"), QStringLiteral("exist Library"));
+    QT_WARNING_POP
+    for (const auto &service : fileItemPlugins) {
+        if (!showGroup.readEntry(service->desktopEntryName(), true)) {
+            // The plugin has been disabled
+            continue;
+        }
+
+        // The plugin also has a JSON metadata and has already been added.
+        if (addedPlugins.contains(service->desktopEntryName())) {
+            continue;
+        }
+
+        KAbstractFileItemActionPlugin *abstractPlugin = m_loadedPlugins.value(service->desktopEntryName());
+        if (!abstractPlugin) {
+            abstractPlugin = service->createInstance<KAbstractFileItemActionPlugin>(this);
+            if (abstractPlugin) {
+                connect(abstractPlugin, &KAbstractFileItemActionPlugin::error, q, &KFileItemActions::error);
+                m_loadedPlugins.insert(service->desktopEntryName(), abstractPlugin);
+                qCWarning(KIO_WIDGETS) << "The" << service->name()
+                                    << "plugin still installs the desktop file for plugin loading. Please use JSON metadata instead, see "
+                                    << "KAbstractFileItemActionPlugin class docs for instructions.";
+            }
+        }
+        if (abstractPlugin) {
+            auto actions = abstractPlugin->actions(m_props, m_parentWidget);
+            itemCount += actions.count();
+            mainMenu->addActions(actions);
+            addedPlugins.append(service->desktopEntryName());
+        }
+    }
+#endif
+
     return itemCount;
 }
 
+KService::List
+KFileItemActionsPrivate::associatedApplications(const QStringList &mimeTypeList, const QString &traderConstraint, const QStringList &excludedDesktopEntryNames)
+{
+    if (!KAuthorized::authorizeAction(QStringLiteral("openwith")) || mimeTypeList.isEmpty()) {
+        return KService::List();
+    }
+
+    KService::List firstOffers;
+    if (traderConstraint.isEmpty()) {
+        firstOffers = KApplicationTrader::queryByMimeType(mimeTypeList.first(), [excludedDesktopEntryNames](const KService::Ptr &service) {
+            return !excludedDesktopEntryNames.contains(service->desktopEntryName());
+        });
+    }
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
+    else {
+        Q_ASSERT(excludedDesktopEntryNames.isEmpty());
+        QT_WARNING_PUSH
+        QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+        QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+        firstOffers = KMimeTypeTrader::self()->query(mimeTypeList.first(), QStringLiteral("Application"), traderConstraint);
+        QT_WARNING_POP
+    }
+#endif
+
+    QList<KFileItemActionsPrivate::ServiceRank> rankings;
+    QStringList serviceList;
+
+    // This section does two things.  First, it determines which services are common to all the given MIME types.
+    // Second, it ranks them based on their preference level in the associated applications list.
+    // The more often a service appear near the front of the list, the LOWER its score.
+
+    rankings.reserve(firstOffers.count());
+    serviceList.reserve(firstOffers.count());
+    for (int i = 0; i < firstOffers.count(); ++i) {
+        KFileItemActionsPrivate::ServiceRank tempRank;
+        tempRank.service = firstOffers[i];
+        tempRank.score = i;
+        rankings << tempRank;
+        serviceList << tempRank.service->storageId();
+    }
+
+    for (int j = 1; j < mimeTypeList.count(); ++j) {
+        KService::List offers;
+        QStringList subservice; // list of services that support this MIME type
+        if (traderConstraint.isEmpty()) {
+            offers = KApplicationTrader::queryByMimeType(mimeTypeList[j], [excludedDesktopEntryNames](const KService::Ptr &service) {
+                return !excludedDesktopEntryNames.contains(service->desktopEntryName());
+            });
+        }
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
+        else {
+            QT_WARNING_PUSH
+            QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+            QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+            Q_ASSERT(excludedDesktopEntryNames.isEmpty());
+            offers = KMimeTypeTrader::self()->query(mimeTypeList[j], QStringLiteral("Application"), traderConstraint);
+            QT_WARNING_POP
+        }
+#endif
+        subservice.reserve(offers.count());
+        for (int i = 0; i != offers.count(); ++i) {
+            const QString serviceId = offers[i]->storageId();
+            subservice << serviceId;
+            const int idPos = serviceList.indexOf(serviceId);
+            if (idPos != -1) {
+                rankings[idPos].score += i;
+            } // else: we ignore the services that didn't support the previous MIME types
+        }
+
+        // Remove services which supported the previous MIME types but don't support this one
+        for (int i = 0; i < serviceList.count(); ++i) {
+            if (!subservice.contains(serviceList[i])) {
+                serviceList.removeAt(i);
+                rankings.removeAt(i);
+                --i;
+            }
+        }
+        // Nothing left -> there is no common application for these MIME types
+        if (rankings.isEmpty()) {
+            return KService::List();
+        }
+    }
+
+    std::sort(rankings.begin(), rankings.end(), KFileItemActionsPrivate::lessRank);
+
+    KService::List result;
+    result.reserve(rankings.size());
+    for (const KFileItemActionsPrivate::ServiceRank &tempRank : qAsConst(rankings)) {
+        result << tempRank.service;
+    }
+
+    return result;
+}
+
+void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
+                                                      QMenu *topMenu,
+                                                      const QStringList &excludedDesktopEntryNames,
+                                                      const QString &traderConstraint)
+{
+    if (!KAuthorized::authorizeAction(QStringLiteral("openwith"))) {
+        return;
+    }
+
+    m_traderConstraint = traderConstraint;
+    // TODO Overload with excludedDesktopEntryNames, but this method in public API and will be handled in a new MR
+    KService::List offers = associatedApplications(m_mimeTypeList, traderConstraint, excludedDesktopEntryNames);
+
+    //// Ok, we have everything, now insert
+
+    const KFileItemList items = m_props.items();
+    const KFileItem &firstItem = items.first();
+    const bool isLocal = firstItem.url().isLocalFile();
+    const bool isDir = m_props.isDirectory();
+    // "Open With..." for folders is really not very useful, especially for remote folders.
+    // (media:/something, or trash:/, or ftp://...).
+    // Don't show "open with" actions for remote dirs only
+    if (isDir && !isLocal) {
+        return;
+    }
+
+    QStringList serviceIdList = listPreferredServiceIds(m_mimeTypeList, excludedDesktopEntryNames, traderConstraint);
+
+    // When selecting files with multiple MIME types, offer either "open with <app for all>"
+    // or a generic <open> (if there are any apps associated).
+    if (m_mimeTypeList.count() > 1 && !serviceIdList.isEmpty()
+        && !(serviceIdList.count() == 1 && serviceIdList.first().isEmpty())) { // empty means "no apps associated"
+
+        QAction *runAct = new QAction(this);
+        if (serviceIdList.count() == 1) {
+            const KService::Ptr app = preferredService(m_mimeTypeList.first(), excludedDesktopEntryNames, traderConstraint);
+            runAct->setText(i18n("&Open with %1", app->name()));
+            runAct->setIcon(QIcon::fromTheme(app->icon()));
+
+            // Remove that app from the offers list (#242731)
+            for (int i = 0; i < offers.count(); ++i) {
+                if (offers[i]->storageId() == app->storageId()) {
+                    offers.removeAt(i);
+                    break;
+                }
+            }
+        } else {
+            runAct->setText(i18n("&Open"));
+        }
+
+        QObject::connect(runAct, &QAction::triggered, this, &KFileItemActionsPrivate::slotRunPreferredApplications);
+        topMenu->insertAction(before, runAct);
+
+        m_traderConstraint = traderConstraint;
+        m_fileOpenList = m_props.items();
+    }
+
+    QAction *openWithAct = new QAction(this);
+    openWithAct->setText(i18nc("@title:menu", "&Open With..."));
+    openWithAct->setObjectName(QStringLiteral("openwith_browse")); // For the unittest
+    QObject::connect(openWithAct, &QAction::triggered, this, &KFileItemActionsPrivate::slotOpenWithDialog);
+
+    if (!offers.isEmpty()) {
+        // Show the top app inline for files, but not folders
+        if (!isDir) {
+            QAction *act = createAppAction(offers.takeFirst(), true);
+            topMenu->insertAction(before, act);
+        }
+
+        // If there are still more apps, show them in a sub-menu
+        if (!offers.isEmpty()) { // submenu 'open with'
+            QMenu *subMenu = new QMenu(i18nc("@title:menu", "&Open With"), topMenu);
+            subMenu->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
+            subMenu->menuAction()->setObjectName(QStringLiteral("openWith_submenu")); // For the unittest
+            // Add other apps to the sub-menu
+            for (const KServicePtr &service : qAsConst(offers)) {
+                QAction *act = createAppAction(service, false);
+                subMenu->addAction(act);
+            }
+
+            subMenu->addSeparator();
+
+            openWithAct->setText(i18nc("@action:inmenu Open With", "&Other Application..."));
+            subMenu->addAction(openWithAct);
+
+            topMenu->insertMenu(before, subMenu);
+            topMenu->insertSeparator(before);
+        } else { // No other apps
+            topMenu->insertAction(before, openWithAct);
+        }
+    } else { // no app offers -> Open With...
+        openWithAct->setIcon(QIcon::fromTheme(QStringLiteral("system-run")));
+        openWithAct->setObjectName(QStringLiteral("openwith")); // For the unittest
+        topMenu->insertAction(before, openWithAct);
+    }
+}
+
+#if KIOWIDGETS_BUILD_DEPRECATED_SINCE(5, 82)
 QAction *KFileItemActions::preferredOpenWithAction(const QString &traderConstraint)
 {
     const KService::List offers = associatedApplications(d->m_mimeTypeList, traderConstraint);
@@ -826,6 +931,7 @@ QAction *KFileItemActions::preferredOpenWithAction(const QString &traderConstrai
     }
     return d->createAppAction(offers.first(), true);
 }
+#endif
 
 void KFileItemActions::setParentWidget(QWidget *widget)
 {
